@@ -49,10 +49,12 @@
 
 politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","average"), drop_blank=FALSE, uk_english=FALSE, num_mc_cores=1){
 
-  text<-unlist(text)
+  text=as.character(unlist(text))
+  text[is.na(text)]<-" "
   if(uk_english){
     text<-usWords(text)
   }
+  text<-paste(text," ")
   ########################################################
   # Generates broad token lists for feature creation below
   if(length(text)<2000){
@@ -77,9 +79,6 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
   ########################################################
   features<-list()
   features[["Hedges"]]<-textcounter(hedge_list,sets[["c.words"]],words=TRUE, num_mc_cores=num_mc_cores)
-  features[["Positive.Emotion"]]<-textcounter(positive_list,sets[["c.words"]],words=TRUE, num_mc_cores=num_mc_cores)
-  features[["Negative.Emotion"]]<-textcounter(negative_list,sets[["c.words"]],words=TRUE, num_mc_cores=num_mc_cores)
-
   features[["Impersonal.Pronoun"]]<-sets[["dicts"]][,"Pronouns"]
   features[["Swearing"]]<-sets[["dicts"]][,"Swearing"]
   features[["Negation"]]<-sets[["dicts"]][,"Negation"]
@@ -87,27 +86,29 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
   features[["Informal.Title"]]<-sets[["dicts"]][,"InformalTitle"]
   features[["Formal.Title"]]<-sets[["dicts"]][,"FormalTitle"]
 
-  features[["Could.You"]]<-textcounter(c("could you","would you"),sets[["clean"]], num_mc_cores=num_mc_cores)
+  features[["Could.You"]]<-textcounter(c("could you","would you","might you"),sets[["clean"]], num_mc_cores=num_mc_cores)
   features[["Can.You"]]<-textcounter(c("can you","will you"),sets[["clean"]], num_mc_cores=num_mc_cores)
 
   features[["By.The.Way"]]<-textcounter(c("by the way"),sets[["clean"]], num_mc_cores=num_mc_cores)
   features[["Let.Me.Know"]]<-textcounter(c("let me know"),sets[["clean"]], num_mc_cores=num_mc_cores)
-  features[["Goodbye"]]<-textcounter(c("goodbye", "bye", "see you later"),sets[["clean"]], num_mc_cores=num_mc_cores)
-  features[["For.Me"]]<-textcounter(c("for me","for us"),sets[["clean"]], num_mc_cores=num_mc_cores)
-  features[["For.You"]]<-textcounter("for you",sets[["clean"]], num_mc_cores=num_mc_cores)
-  features[["Reasoning"]]<-textcounter(c("reason", "why i", "why we", "explain", "caused","because"),sets[["clean"]],
+  features[["Goodbye"]]<-textcounter(c("goodbye", " bye ", "see you later"),sets[["clean"]], num_mc_cores=num_mc_cores)
+  features[["For.Me"]]<-textcounter(c(" for me ","for us "),sets[["clean"]], num_mc_cores=num_mc_cores)
+  features[["For.You"]]<-textcounter(" for you ",sets[["clean"]], num_mc_cores=num_mc_cores)
+  features[["Reasoning"]]<-textcounter(c("reason", "why i ", "why we ", "explain", "caused","because"),sets[["clean"]],
                                        num_mc_cores=num_mc_cores)
   features[["Reassurance"]]<-textcounter(c("is okay", "not worry", "no big deal", "not a big deal", "no problem",
                                            "no worries", "is fine", "you are good", "it's fine", "it's okay") ,sets[["clean"]],
                                          num_mc_cores=num_mc_cores)
-  features[["Ask.Agency"]]<-textcounter(c("do me a favor", "do me a favour", "let me", "allow me", "can i", "should i",
-                                          "may i", "might i", "could i"),sets[["clean"]],
+  features[["Ask.Agency"]]<-textcounter(c("do me a favor", " let me ", " allow me ", " can i ", " should i ",
+                                          " may i ", "might i ", " could i "),sets[["clean"]],
                                         num_mc_cores=num_mc_cores)
-  features[["Give.Agency"]]<-textcounter(c("let you", "allow you", "you can", "you may", "you could"),sets[["clean"]],
+  features[["Give.Agency"]]<-textcounter(c("let you", "allow you", "you can ", " you may ", " you could "),sets[["clean"]],
                                          num_mc_cores=num_mc_cores)
 
-  features[["Hello"]]<-textcounter(c("hi","hello","hey"),sets[["c.words"]],words=TRUE,
-                                   num_mc_cores=num_mc_cores)  # "good morning", "good evening", "good afternoon",
+  features[["Hello"]]<-(textcounter(c("hi","hello","hey","greetings"),sets[["c.words"]],words=TRUE,
+                                    num_mc_cores=num_mc_cores)+
+                          textcounter(c("good morning", "good evening", "good afternoon"),sets[["clean"]],
+                                      num_mc_cores=num_mc_cores))
   features[["Please"]]<-1*(grepl("please",sets[["c.words"]],fixed=TRUE))
 
   features[["First.Person.Plural"]]<-textcounter(c("we", "our", "ours", "us", "ourselves"),sets[["c.words"]],words=TRUE,
@@ -119,6 +120,8 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
   #if(parser[1]=="none"){
   if(parser[1]!="spacy"){
     cat("Note: Some features cannot be computed without part-of-speech tagging. See ?spacyr::spacyr for details.")
+    features[["Positive.Emotion"]]<-textcounter(positive_list,sets[["c.words"]],words=TRUE, num_mc_cores=num_mc_cores)
+    features[["Negative.Emotion"]]<-textcounter(negative_list,sets[["c.words"]],words=TRUE, num_mc_cores=num_mc_cores)
     features[["Questions"]]<-textcounter("?",text, num_mc_cores=num_mc_cores)
     features[["Gratitude"]]<-(unlist(lapply(sets[["c.words"]], function(x) sum(startsWith(unlist(x), prefix="thank"))))+
                                 unlist(lapply(sets[["c.words"]], function(x) sum(startsWith(unlist(x), prefix="grateful"))))+
@@ -129,23 +132,56 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
                                                   num_mc_cores=num_mc_cores)+
                                         textcounter(c("in fact"),sets[["clean"]],num_mc_cores=num_mc_cores))
   } else {
-    # SLOW POKE!!
-    features[["Agreement"]]<-(unlist(lapply(sets[["p.nonum"]],function(x) sum(textcounter(c("nsubj(agree, i)","nsubj(concur, i)",
-                                                                                            "nsubj(agree, we)","nsubj(concur, we)",
-                                                                                            "acomp('re, right)","acomp(are, right)"),x, words=TRUE,
-                                                                                          num_mc_cores=num_mc_cores)-
-                                                                                textcounter(c("neg(agree","neg(concur"),x,
-                                                                                            num_mc_cores=num_mc_cores))))+
-                                textcounter(apply(expand.grid(c("good","great","excellent"),c("idea", "point")),1,paste, collapse=" "),sets[["clean"]],num_mc_cores=num_mc_cores))
-    # SLOW POKE!!
-    features[["Acknowledgement"]]<-unlist(lapply(sets[["p.nonum"]],function(x) sum(textcounter(c("nsubj(understand, i)","nsubj(see, i)","nsubj(acknowledge, i)",
-                                                                                                 "nsubj(hear, i)","nsubj(get, i)",
-                                                                                                 "nsubj(understand, we)","nsubj(see, we)","nsubj(acknowledge, we)",
-                                                                                                 "nsubj(hear, we)","nsubj(get, we)"),x, words=TRUE,
-                                                                                               num_mc_cores=num_mc_cores)-
-                                                                                     textcounter(c("neg(understand","neg(see","neg(get"),x,
-                                                                                                 num_mc_cores=num_mc_cores))))
-    # SLOW POKE!!
+    features[["Positive.Emotion"]]<-(textcounter(positive_list,sets[["unneg.words"]],words=TRUE, num_mc_cores=num_mc_cores)
+                                     +textcounter(negative_list,sets[["neg.words"]],words=TRUE, num_mc_cores=num_mc_cores))
+    features[["Negative.Emotion"]]<-(textcounter(positive_list,sets[["neg.words"]],words=TRUE, num_mc_cores=num_mc_cores)
+                                     +textcounter(negative_list,sets[["unneg.words"]],words=TRUE, num_mc_cores=num_mc_cores))
+
+    features[["Agreement"]]<-(unlist(lapply(sets[["p.unnegs"]],
+                                            function(x) sum(textcounter(c("nsubj(agree, i)","nsubj(concur, i)",
+                                                                          "nsubj(agree, we)","nsubj(concur, we)",
+                                                                          "acomp('re, right)","acomp(are, right)",
+                                                                          "acomp('re, correct)","acomp(are, correct)",
+                                                                          "acomp('s, right)","acomp(is, right)",
+                                                                          "acomp('s, correct)","acomp(is, correct)",
+                                                                          "acomp('s, true)","acomp(is, true)"),x, words=TRUE,
+                                                                        num_mc_cores=num_mc_cores))))+
+                                textcounter(apply(expand.grid(c("good","great","excellent","brilliant","fair","amazing"),
+                                                              c("idea", "point","suggestion")),1,paste, collapse=" "),
+                                            sets[["clean"]],num_mc_cores=num_mc_cores))
+    features[["Disagreement"]]<-(unlist(lapply(sets[["p.negs"]],
+                                               function(x) sum(textcounter(c("nsubj(agree, i)","nsubj(concur, i)",
+                                                                             "nsubj(agree, we)","nsubj(concur, we)",
+                                                                             "nsubj(think, we)","nsubj(think, i)",
+                                                                             "nsubj(believe, we)","nsubj(believe, i)",
+                                                                             "acomp('re, right)","acomp(are, right)",
+                                                                             "acomp('re, correct)","acomp(are, correct)",
+                                                                             "acomp('s, right)","acomp(is, right)",
+                                                                             "acomp('s, correct)","acomp(is, correct)",
+                                                                             "acomp('s, true)","acomp(is, true)"),x, words=TRUE,
+                                                                           num_mc_cores=num_mc_cores))))
+                                 +unlist(lapply(sets[["p.unnegs"]],
+                                                function(x) sum(textcounter(c("nsubj(disagree, i)","nsubj(object, i)",
+                                                                              "nsubj(disagree, we)","nsubj(object, we)",
+                                                                              "acomp('re, wrong)","acomp(are, wrong)",
+                                                                              "acomp('re, incorrect)","acomp(are, incorrect)",
+                                                                              "acomp('s, wrong)","acomp(is, wrong)",
+                                                                              "acomp('s, incorrect)","acomp(is, incorrect)",
+                                                                              "acomp('s, untrue)","acomp(is, untrue)",
+                                                                              "det(lie, a)","det(myth, a)",
+                                                                              "acomp('s, false)","acomp(is, false)"),x, words=TRUE,
+                                                                            num_mc_cores=num_mc_cores))))
+                                 +textcounter(apply(expand.grid(c("bad","terrible","horrible","awful","dumb","stupid"),
+                                                                c("idea", "point","suggestion")),1,paste, collapse=" "),
+                                              sets[["clean"]],num_mc_cores=num_mc_cores)
+    )
+
+    features[["Acknowledgement"]]<-unlist(lapply(sets[["p.unnegs"]],function(x) sum(textcounter(c("nsubj(understand, i)","nsubj(see, i)","nsubj(acknowledge, i)",
+                                                                                                  "nsubj(hear, i)","nsubj(get, i)","dobj(take, point)",
+                                                                                                  "nsubj(understand, we)","nsubj(see, we)","nsubj(acknowledge, we)",
+                                                                                                  "nsubj(hear, we)","nsubj(get, we)"),x, words=TRUE,
+                                                                                                num_mc_cores=num_mc_cores))))
+
     features[["Subjectivity"]]<-unlist(lapply(sets[["p.nonum"]],function(x) sum(textcounter(c("nsubj(think, i)","nsubj(believe, i)","nsubj(suspect, i)",
                                                                                               "nsubj(thought, i)","nsubj(felt, i)",
                                                                                               "nsubj(presume, i)","nsubj(reckon, i)","nsubj(feel, i)",
@@ -156,42 +192,56 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
                                                                                               "nsubj(presume, we)","nsubj(reckon, we)","nsubj(feel, we)",
                                                                                               "poss(perspective, our)","poss(belief, our)","poss(view, our)",
                                                                                               "poss(knowledge, our)","poss(opinion, our)"),x, words=TRUE,
-                                                                                               num_mc_cores=num_mc_cores))))
+                                                                                            num_mc_cores=num_mc_cores))))
 
-    # SLOW POKE!!
-    features[["Bare.Command"]]<-unlist(lapply(sets[["pos.nums"]],function(x) sum(grepl("(1-",unlist(x),fixed=TRUE)&grepl("-vb)",unlist(x),fixed=TRUE)
-                                                                                 &(textcounter(paste0("-",c("be","do","have","thank","please","hang","let"),"-"),
-                                                                                               unlist(x),fixed=T)==0))))
 
+    features[["Bare.Command"]]<-unlist(lapply(sets[["pos.nums"]],function(x) sum(grepl("(1-",unlist(x),fixed=TRUE)
+                                                                                 &grepl("-vb)",unlist(x),fixed=TRUE)
+                                                                                 &(!grepl(paste0("-be-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-do-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-have-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-thank-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-please-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-hang-"),unlist(x)))
+                                                                                 &(!grepl(paste0("-let-"),unlist(x)))
+    )))
+
+     q.words<-c("who","what","where","when","why","how","which")
+    features[["WH.Questions"]]<-unlist(lapply(sets[["ques.pos.dists"]],
+                                               function(x) sum(textcounter(c(paste0(q.words,"-wrb"),paste0(q.words,"-wdt"),paste0(q.words,"-wp")),x))))
+    # features[["WH.QuestionsX"]]<-unlist(lapply(sets[["ques.pos.dists"]],
+    #                                           function(x) sum(textcounter(c("wrb)-0","wdt)-0","-wp)-0","wrb)-1","wdt)-1","-wp)-1"),x))))
+    # maybe look for all words before the root? except...
+    # "At what point do you say enough is enough?"
+    # "Given what has happened, do you feel that the actions in response to this were always appropriate?"
+    features[["YesNo.Questions"]]<-unlist(lapply(sets[["ques.pos.dists"]],
+                                                 function(x) sum(textcounter("-?-",x,num_mc_cores=num_mc_cores))))-features[["WH.Questions"]]
+    # Also....
     # Tag Questions cases like "right?" and "don't you?", "eh?", "you know?" "what do you think?"
     # Repair Questions	(from SpeedDate)? "pardon?" "sorry?"
-    q.words<-c("who","what","where","when","why","how","which")
-    features[["WH.Questions"]]<-unlist(lapply(sets[["ques.pos.nums"]],
-                                              function(x) sum(textcounter(c(paste0(q.words,"-wrb")),x))))
-
-    features[["YesNo.Questions"]]<-unlist(lapply(sets[["ques.pos.nums"]],
-                                                 function(x) sum(textcounter("-?-",x,num_mc_cores=num_mc_cores))))-features[["WH.Questions"]]
-
     features[["Gratitude"]]<-(unlist(lapply(sets[["c.words"]], function(x) sum(startsWith(unlist(x), prefix="thank"))))+
                                 unlist(lapply(sets[["c.words"]], function(x) sum(startsWith(unlist(x), prefix="grateful"))))+
                                 unlist(lapply(sets[["c.words"]], function(x) sum(startsWith(unlist(x), prefix="gratitude"))))+
-                                unlist(lapply(sets[["p.nonum"]], function(x) sum(grepl("(appreciate, we)",x,fixed=TRUE))))+
-                                unlist(lapply(sets[["p.nonum"]], function(x) sum(grepl("(appreciate, i)",x,fixed=TRUE)))))
-    features[["Apology"]]<-(textcounter(c("sorry"," woops","oops","whoops"),sets[["c.words"]],words=TRUE,
+                                unlist(lapply(sets[["p.unnegs"]], function(x) sum(grepl("(appreciate, we)",x,fixed=TRUE))))+
+                                unlist(lapply(sets[["p.unnegs"]], function(x) sum(grepl("(appreciate, i)",x,fixed=TRUE)))))
+    features[["Apology"]]<-(textcounter(c("sorry"," woops","oops","whoops"),sets[["unneg.words"]],words=TRUE,
                                         num_mc_cores=num_mc_cores)
-                            +textcounter(c("dobj(excuse, me)","nsubj(apologize, we)","nsubj(apologize, i)","dobj(forgive, me)"),sets[["p.nonum"]], words=TRUE,
+                            +textcounter(c("dobj(excuse, me)","nsubj(apologize, we)","nsubj(apologize, i)","dobj(forgive, me)"),sets[["p.unnegs"]], words=TRUE,
                                          num_mc_cores=num_mc_cores)
-                            -textcounter(c("not sorry"),sets[["clean"]],num_mc_cores=num_mc_cores)
-                            -textcounter(c("neg(apologize"),sets[["p.nonum"]],num_mc_cores=num_mc_cores)
-                            )
+    )
     features[["Truth.Intensifier"]]<-(textcounter(c("really", "actually", "honestly", "surely"),sets[["c.words"]],words=TRUE,
                                                   num_mc_cores=num_mc_cores)
-                                      +textcounter(c("det(point, the)","det(reality, the)","det(truth, the)","case(fact, in)"),sets[["p.nonum"]], words=TRUE,
+                                      +textcounter(c("det(point, the)","det(reality, the)","det(truth, the)","pobj(fact, in)","case(fact, in)"),sets[["p.nonum"]], words=TRUE,
                                                    num_mc_cores=num_mc_cores))
-    features[["Affirmation"]]<-textcounter(paste0(c("yeah","yes","ok","okay","perfect","fine","wow","great",
+    features[["Affirmation"]]<-textcounter(paste0(c("yeah","yes","ok","okay","perfect","fine","wow","great","amazing","fantastic",
                                                     "good","nice","interesting","cool","excellent","awesome"),"-1"),sets[["w.nums"]],words=TRUE,
                                            num_mc_cores=num_mc_cores)
-    features[["Adverb.Just"]]<-unlist(lapply(sets[["p.nonum"]] ,function(x) sum(grepl("advmod",unlist(x))&grepl("just)",unlist(x),fixed=TRUE))))
+    features[["Adverb.Limiter"]]<-unlist(lapply(sets[["p.nonum"]] ,function(x) sum(grepl("advmod",unlist(x))&
+                                                                                     (grepl("just)",unlist(x),fixed=TRUE)
+                                                                                      |grepl("only)",unlist(x),fixed=TRUE)
+                                                                                      |grepl("merely)",unlist(x),fixed=TRUE)
+                                                                                      |grepl("simply)",unlist(x),fixed=TRUE)))
+    ))
 
     features[["Conjunction.Start"]]<-textcounter(paste0(c("so","then","and","but","or"),"-1"),sets[["w.nums"]],words=TRUE,
                                                  num_mc_cores=num_mc_cores)
@@ -199,8 +249,7 @@ politeness<-function(text, parser=c("none","spacy"), metric=c("count","binary","
 
   if(metric[1]=="binary"){
     features<-parallel::mclapply(features, function(x) 1*(x>0), mc.cores=num_mc_cores)
-  }
-  else if (metric[1]=="average"){
+  } else if (metric[1]=="average"){
     word_counts <- stringr::str_count(text, "[[:alpha:]]+")
     features<-parallel::mclapply(features, function(x) x/word_counts, mc.cores=num_mc_cores)
   }
